@@ -2,6 +2,7 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
+from __future__ import annotations
 
 INCLUDES = """
 #include <openssl/asn1.h>
@@ -39,20 +40,8 @@ static const int MBSTRING_UTF8;
 """
 
 FUNCTIONS = """
-void ASN1_OBJECT_free(ASN1_OBJECT *);
-
 /*  ASN1 STRING */
-unsigned char *ASN1_STRING_data(ASN1_STRING *);
 const unsigned char *ASN1_STRING_get0_data(const ASN1_STRING *);
-int ASN1_STRING_set(ASN1_STRING *, const void *, int);
-
-/*  ASN1 OCTET STRING */
-ASN1_OCTET_STRING *ASN1_OCTET_STRING_new(void);
-void ASN1_OCTET_STRING_free(ASN1_OCTET_STRING *);
-int ASN1_OCTET_STRING_set(ASN1_OCTET_STRING *, const unsigned char *, int);
-
-/* ASN1 IA5STRING */
-ASN1_IA5STRING *ASN1_IA5STRING_new(void);
 
 /*  ASN1 INTEGER */
 void ASN1_INTEGER_free(ASN1_INTEGER *);
@@ -71,17 +60,14 @@ ASN1_ENUMERATED *ASN1_ENUMERATED_new(void);
 void ASN1_ENUMERATED_free(ASN1_ENUMERATED *);
 int ASN1_ENUMERATED_set(ASN1_ENUMERATED *, long);
 
-/* These became const ASN1_* in 1.1.0 */
-int ASN1_STRING_type(ASN1_STRING *);
-int ASN1_STRING_to_UTF8(unsigned char **, ASN1_STRING *);
-int i2a_ASN1_INTEGER(BIO *, ASN1_INTEGER *);
+int ASN1_STRING_type(const ASN1_STRING *);
+int ASN1_STRING_to_UTF8(unsigned char **, const ASN1_STRING *);
+int i2a_ASN1_INTEGER(BIO *, const ASN1_INTEGER *);
 
-/* This became const ASN1_TIME in 1.1.0f */
-ASN1_GENERALIZEDTIME *ASN1_TIME_to_generalizedtime(ASN1_TIME *,
+ASN1_GENERALIZEDTIME *ASN1_TIME_to_generalizedtime(const ASN1_TIME *,
                                                    ASN1_GENERALIZEDTIME **);
 
 int ASN1_STRING_length(ASN1_STRING *);
-int ASN1_STRING_set_default_mask_asc(char *);
 
 BIGNUM *ASN1_INTEGER_to_BN(ASN1_INTEGER *, BIGNUM *);
 ASN1_INTEGER *BN_to_ASN1_INTEGER(BIGNUM *, ASN1_INTEGER *);
