@@ -2,7 +2,7 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
+from __future__ import annotations
 
 INCLUDES = """
 #include <openssl/dsa.h>
@@ -23,11 +23,7 @@ int DSA_sign(int, const unsigned char *, int, unsigned char *, unsigned int *,
 int DSA_verify(int, const unsigned char *, int, const unsigned char *, int,
                DSA *);
 
-/* added in 1.1.0 to access the opaque struct */
-void DSA_get0_pqg(const DSA *, const BIGNUM **, const BIGNUM **,
-                  const BIGNUM **);
 int DSA_set0_pqg(DSA *, BIGNUM *, BIGNUM *, BIGNUM *);
-void DSA_get0_key(const DSA *, const BIGNUM **, const BIGNUM **);
 int DSA_set0_key(DSA *, BIGNUM *, BIGNUM *);
 int DSA_generate_parameters_ex(DSA *, int, unsigned char *, int,
                                int *, unsigned long *, BN_GENCB *);
