@@ -17,13 +17,13 @@ from cryptography.hazmat.primitives.ciphers.aead import (
     ChaCha20Poly1305,
 )
 
-from .utils import _load_all_params
 from ...utils import (
     load_nist_ccm_vectors,
     load_nist_vectors,
     load_vectors_from_file,
     raises_unsupported_algorithm,
 )
+from .utils import _load_all_params
 
 
 class FakeData(bytes):
@@ -464,10 +464,22 @@ class TestAESGCM:
         computed_pt = aesgcm.decrypt(nonce, ct, ad)
         assert computed_pt == pt
         aesgcm2 = AESGCM(bytearray(key))
-        ct2 = aesgcm2.encrypt(bytearray(nonce), pt, ad)
+        ct2 = aesgcm2.encrypt(bytearray(nonce), bytearray(pt), bytearray(ad))
         assert ct2 == ct
-        computed_pt2 = aesgcm2.decrypt(bytearray(nonce), ct2, ad)
+        b_nonce = bytearray(nonce)
+        b_ct2 = bytearray(ct2)
+        b_ad = bytearray(ad)
+        computed_pt2 = aesgcm2.decrypt(b_nonce, b_ct2, b_ad)
         assert computed_pt2 == pt
+        aesgcm3 = AESGCM(memoryview(key))
+        m_nonce = memoryview(nonce)
+        m_pt = memoryview(pt)
+        m_ad = memoryview(ad)
+        ct3 = aesgcm3.encrypt(m_nonce, m_pt, m_ad)
+        assert ct3 == ct
+        m_ct3 = memoryview(ct3)
+        computed_pt3 = aesgcm3.decrypt(m_nonce, m_ct3, m_ad)
+        assert computed_pt3 == pt
 
 
 @pytest.mark.skipif(
@@ -616,7 +628,7 @@ class TestAESOCB3:
     not _aead_supported(AESSIV),
     reason="Does not support AESSIV",
 )
-class TestAESSIV(object):
+class TestAESSIV:
     def test_data_too_large(self):
         key = AESSIV.generate_key(256)
         aessiv = AESSIV(key)
@@ -625,7 +637,17 @@ class TestAESSIV(object):
             aessiv.encrypt(FakeData(), None)
 
         with pytest.raises(OverflowError):
-            aessiv.encrypt(b"", [FakeData()])
+            aessiv.encrypt(b"irrelevant", [FakeData()])
+
+    def test_no_empty_encryption(self):
+        key = AESSIV.generate_key(256)
+        aessiv = AESSIV(key)
+
+        with pytest.raises(ValueError):
+            aessiv.encrypt(b"", None)
+
+        with pytest.raises(ValueError):
+            aessiv.decrypt(b"", None)
 
     def test_vectors(self, backend, subtests):
         vectors = load_vectors_from_file(
