@@ -2,18 +2,16 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import binascii
 import os
 
 import pytest
 
-from cryptography.hazmat.backends.interfaces import CipherBackend
 from cryptography.hazmat.primitives.ciphers import algorithms
 
-from .utils import generate_stream_encryption_test
 from ...utils import load_nist_vectors
+from .utils import generate_stream_encryption_test
 
 
 @pytest.mark.supported(
@@ -22,8 +20,7 @@ from ...utils import load_nist_vectors
     ),
     skip_message="Does not support ARC4",
 )
-@pytest.mark.requires_backend_interface(interface=CipherBackend)
-class TestARC4(object):
+class TestARC4:
     test_rfc = generate_stream_encryption_test(
         load_nist_vectors,
         os.path.join("ciphers", "ARC4"),
