@@ -7,12 +7,7 @@ import binascii
 
 import pytest
 
-from cryptography.exceptions import (
-    AlreadyFinalized,
-    InvalidSignature,
-    _Reasons,
-)
-from cryptography.hazmat.backends.interfaces import CMACBackend
+from cryptography.exceptions import AlreadyFinalized, InvalidSignature
 from cryptography.hazmat.primitives.ciphers.algorithms import (
     AES,
     ARC4,
@@ -20,12 +15,7 @@ from cryptography.hazmat.primitives.ciphers.algorithms import (
 )
 from cryptography.hazmat.primitives.cmac import CMAC
 
-from ...utils import (
-    load_nist_vectors,
-    load_vectors_from_file,
-    raises_unsupported_algorithm,
-)
-
+from ...utils import load_nist_vectors, load_vectors_from_file
 
 vectors_aes128 = load_vectors_from_file(
     "CMAC/nist-800-38b-aes128.txt", load_nist_vectors
@@ -48,8 +38,7 @@ vectors_3des = load_vectors_from_file(
 fake_key = b"\x00" * 16
 
 
-@pytest.mark.requires_backend_interface(interface=CMACBackend)
-class TestCMAC(object):
+class TestCMAC:
     @pytest.mark.supported(
         only_if=lambda backend: backend.cmac_algorithm_supported(
             AES(fake_key)
@@ -212,11 +201,3 @@ class TestCMAC(object):
         assert cmac.finalize() == binascii.unhexlify(
             b"a21e6e647bfeaf5ca0a5e1bcd957dfad"
         )
-
-
-def test_invalid_backend():
-    key = b"2b7e151628aed2a6abf7158809cf4f3c"
-    pretend_backend = object()
-
-    with raises_unsupported_algorithm(_Reasons.BACKEND_MISSING_INTERFACE):
-        CMAC(AES(key), pretend_backend)
