@@ -9,54 +9,58 @@ import typing
 
 import pytest
 
-from cryptography.exceptions import AlreadyFinalized, InvalidSignature
-from cryptography.hazmat.backends.interfaces import (
-    DSABackend,
-    PEMSerializationBackend,
-)
+from cryptography import utils
+from cryptography.exceptions import InvalidSignature
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import dsa
 from cryptography.hazmat.primitives.asymmetric.utils import (
     Prehashed,
     encode_dss_signature,
 )
-from cryptography.utils import CryptographyDeprecationWarning
 
-from .fixtures_dsa import DSA_KEY_1024, DSA_KEY_2048, DSA_KEY_3072
-from .utils import skip_fips_traditional_openssl
 from ...doubles import DummyHashAlgorithm, DummyKeySerializationEncryption
 from ...utils import (
     load_fips_dsa_key_pair_vectors,
     load_fips_dsa_sig_vectors,
     load_vectors_from_file,
 )
+from .fixtures_dsa import DSA_KEY_1024, DSA_KEY_2048, DSA_KEY_3072
+from .utils import skip_fips_traditional_openssl
 
-_ALGORITHMS_DICT: typing.Dict[str, typing.Type[hashes.HashAlgorithm]] = {
-    "SHA1": hashes.SHA1,
-    "SHA224": hashes.SHA224,
-    "SHA256": hashes.SHA256,
-    "SHA384": hashes.SHA384,
-    "SHA512": hashes.SHA512,
+_ALGORITHMS_DICT: typing.Dict[str, hashes.HashAlgorithm] = {
+    "SHA1": hashes.SHA1(),
+    "SHA224": hashes.SHA224(),
+    "SHA256": hashes.SHA256(),
+    "SHA384": hashes.SHA384(),
+    "SHA512": hashes.SHA512(),
 }
 
 
-def _skip_if_dsa_not_supported(backend, algorithm, p, q, g):
-    if not backend.dsa_parameters_supported(
-        p, q, g
-    ) or not backend.dsa_hash_supported(algorithm):
+def _skip_if_dsa_not_supported(
+    backend: typing.Any,
+    algorithm: hashes.HashAlgorithm,
+    p: int,
+    q: int,
+    g: int,
+) -> None:
+    if not backend.dsa_hash_supported(algorithm):
         pytest.skip(
-            "{} does not support the provided parameters".format(backend)
+            "{} does not support the provided args. p: {}, hash: {}".format(
+                backend, p.bit_length(), algorithm.name
+            )
         )
 
 
-@pytest.mark.requires_backend_interface(interface=DSABackend)
 def test_skip_if_dsa_not_supported(backend):
     with pytest.raises(pytest.skip.Exception):
         _skip_if_dsa_not_supported(backend, DummyHashAlgorithm(), 1, 1, 1)
 
 
-@pytest.mark.requires_backend_interface(interface=DSABackend)
-class TestDSA(object):
+@pytest.mark.supported(
+    only_if=lambda backend: backend.dsa_supported(),
+    skip_message="Does not support DSA.",
+)
+class TestDSA:
     def test_generate_dsa_parameters(self, backend):
         parameters = dsa.generate_parameters(2048, backend)
         assert isinstance(parameters, dsa.DSAParameters)
@@ -73,11 +77,6 @@ class TestDSA(object):
         ),
     )
     def test_generate_dsa_keys(self, vector, backend):
-        if (
-            backend._fips_enabled
-            and vector["p"] < backend._fips_dsa_min_modulus
-        ):
-            pytest.skip("Small modulus blocked in FIPS mode")
         parameters = dsa.DSAParameterNumbers(
             p=vector["p"], q=vector["q"], g=vector["g"]
         ).parameters(backend)
@@ -114,38 +113,38 @@ class TestDSA(object):
         ("p", "q", "g"),
         [
             (
-                2 ** 1000,
+                2**1000,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
             ),
             (
-                2 ** 2000,
+                2**2000,
                 DSA_KEY_2048.public_numbers.parameter_numbers.q,
                 DSA_KEY_2048.public_numbers.parameter_numbers.g,
             ),
             (
-                2 ** 3000,
+                2**3000,
                 DSA_KEY_3072.public_numbers.parameter_numbers.q,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
             ),
             (
-                2 ** 3100,
+                2**3100,
                 DSA_KEY_3072.public_numbers.parameter_numbers.q,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
             ),
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
-                2 ** 150,
+                2**150,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
             ),
             (
                 DSA_KEY_2048.public_numbers.parameter_numbers.p,
-                2 ** 250,
+                2**250,
                 DSA_KEY_2048.public_numbers.parameter_numbers.g,
             ),
             (
                 DSA_KEY_3072.public_numbers.parameter_numbers.p,
-                2 ** 260,
+                2**260,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
             ),
             (
@@ -161,7 +160,7 @@ class TestDSA(object):
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
-                2 ** 1200,
+                2**1200,
             ),
         ],
     )
@@ -173,28 +172,28 @@ class TestDSA(object):
         ("p", "q", "g", "y", "x"),
         [
             (
-                2 ** 1000,
+                2**1000,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
                 DSA_KEY_1024.public_numbers.y,
                 DSA_KEY_1024.x,
             ),
             (
-                2 ** 2000,
+                2**2000,
                 DSA_KEY_2048.public_numbers.parameter_numbers.q,
                 DSA_KEY_2048.public_numbers.parameter_numbers.g,
                 DSA_KEY_2048.public_numbers.y,
                 DSA_KEY_2048.x,
             ),
             (
-                2 ** 3000,
+                2**3000,
                 DSA_KEY_3072.public_numbers.parameter_numbers.q,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
                 DSA_KEY_3072.public_numbers.y,
                 DSA_KEY_3072.x,
             ),
             (
-                2 ** 3100,
+                2**3100,
                 DSA_KEY_3072.public_numbers.parameter_numbers.q,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
                 DSA_KEY_3072.public_numbers.y,
@@ -202,21 +201,21 @@ class TestDSA(object):
             ),
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
-                2 ** 150,
+                2**150,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
                 DSA_KEY_1024.public_numbers.y,
                 DSA_KEY_1024.x,
             ),
             (
                 DSA_KEY_2048.public_numbers.parameter_numbers.p,
-                2 ** 250,
+                2**250,
                 DSA_KEY_2048.public_numbers.parameter_numbers.g,
                 DSA_KEY_2048.public_numbers.y,
                 DSA_KEY_2048.x,
             ),
             (
                 DSA_KEY_3072.public_numbers.parameter_numbers.p,
-                2 ** 260,
+                2**260,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
                 DSA_KEY_3072.public_numbers.y,
                 DSA_KEY_3072.x,
@@ -238,7 +237,7 @@ class TestDSA(object):
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
-                2 ** 1200,
+                2**1200,
                 DSA_KEY_1024.public_numbers.y,
                 DSA_KEY_1024.x,
             ),
@@ -261,20 +260,20 @@ class TestDSA(object):
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
                 DSA_KEY_1024.public_numbers.y,
-                2 ** 159,
+                2**159,
             ),
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
                 DSA_KEY_1024.public_numbers.y,
-                2 ** 200,
+                2**200,
             ),
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
-                2 ** 100,
+                2**100,
                 DSA_KEY_1024.x,
             ),
         ],
@@ -293,44 +292,44 @@ class TestDSA(object):
         ("p", "q", "g", "y"),
         [
             (
-                2 ** 1000,
+                2**1000,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
                 DSA_KEY_1024.public_numbers.y,
             ),
             (
-                2 ** 2000,
+                2**2000,
                 DSA_KEY_2048.public_numbers.parameter_numbers.q,
                 DSA_KEY_2048.public_numbers.parameter_numbers.g,
                 DSA_KEY_2048.public_numbers.y,
             ),
             (
-                2 ** 3000,
+                2**3000,
                 DSA_KEY_3072.public_numbers.parameter_numbers.q,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
                 DSA_KEY_3072.public_numbers.y,
             ),
             (
-                2 ** 3100,
+                2**3100,
                 DSA_KEY_3072.public_numbers.parameter_numbers.q,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
                 DSA_KEY_3072.public_numbers.y,
             ),
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
-                2 ** 150,
+                2**150,
                 DSA_KEY_1024.public_numbers.parameter_numbers.g,
                 DSA_KEY_1024.public_numbers.y,
             ),
             (
                 DSA_KEY_2048.public_numbers.parameter_numbers.p,
-                2 ** 250,
+                2**250,
                 DSA_KEY_2048.public_numbers.parameter_numbers.g,
                 DSA_KEY_2048.public_numbers.y,
             ),
             (
                 DSA_KEY_3072.public_numbers.parameter_numbers.p,
-                2 ** 260,
+                2**260,
                 DSA_KEY_3072.public_numbers.parameter_numbers.g,
                 DSA_KEY_3072.public_numbers.y,
             ),
@@ -349,7 +348,7 @@ class TestDSA(object):
             (
                 DSA_KEY_1024.public_numbers.parameter_numbers.p,
                 DSA_KEY_1024.public_numbers.parameter_numbers.q,
-                2 ** 1200,
+                2**1200,
                 DSA_KEY_1024.public_numbers.y,
             ),
         ],
@@ -368,6 +367,7 @@ class TestDSA(object):
             ),
             mode="rb",
         )
+        assert isinstance(key, dsa.DSAPrivateKey)
         pn = key.private_numbers()
         assert pn.public_numbers.parameter_numbers.p.bit_length() == 4096
         # Turn it back into a key to confirm that values this large pass
@@ -384,9 +384,26 @@ class TestDSA(object):
             x=pn.x,
         ).private_key(backend)
 
+    def test_public_key_equality(self, backend):
+        key_bytes = load_vectors_from_file(
+            os.path.join("asymmetric", "PKCS8", "unenc-dsa-pkcs8.pem"),
+            lambda pemfile: pemfile.read().encode(),
+        )
+        key1 = serialization.load_pem_private_key(key_bytes, None).public_key()
+        key2 = serialization.load_pem_private_key(key_bytes, None).public_key()
+        key3 = DSA_KEY_2048.private_key().public_key()
+        assert key1 == key2
+        assert key1 != key3
+        assert key1 != object()
+        with pytest.raises(TypeError):
+            key1 < key2  # type: ignore[operator]
 
-@pytest.mark.requires_backend_interface(interface=DSABackend)
-class TestDSAVerification(object):
+
+@pytest.mark.supported(
+    only_if=lambda backend: backend.dsa_supported(),
+    skip_message="Does not support DSA.",
+)
+class TestDSAVerification:
     def test_dsa_verification(self, backend, subtests):
         vectors = load_vectors_from_file(
             os.path.join("asymmetric", "DSA", "FIPS_186-3", "SigVer.rsp"),
@@ -411,34 +428,15 @@ class TestDSAVerification(object):
 
                 if vector["result"] == "F":
                     with pytest.raises(InvalidSignature):
-                        public_key.verify(sig, vector["msg"], algorithm())
+                        public_key.verify(sig, vector["msg"], algorithm)
                 else:
-                    public_key.verify(sig, vector["msg"], algorithm())
+                    public_key.verify(sig, vector["msg"], algorithm)
 
     def test_dsa_verify_invalid_asn1(self, backend):
         public_key = DSA_KEY_1024.public_numbers.public_key(backend)
         with pytest.raises(InvalidSignature):
             public_key.verify(b"fakesig", b"fakemsg", hashes.SHA1())
 
-    def test_signature_not_bytes(self, backend):
-        public_key = DSA_KEY_1024.public_numbers.public_key(backend)
-        with pytest.raises(TypeError), pytest.warns(
-            CryptographyDeprecationWarning
-        ):
-            public_key.verifier(1234, hashes.SHA1())
-
-    def test_use_after_finalize(self, backend):
-        public_key = DSA_KEY_1024.public_numbers.public_key(backend)
-        with pytest.warns(CryptographyDeprecationWarning):
-            verifier = public_key.verifier(b"fakesig", hashes.SHA1())
-        verifier.update(b"irrelevant")
-        with pytest.raises(InvalidSignature):
-            verifier.verify()
-        with pytest.raises(AlreadyFinalized):
-            verifier.verify()
-        with pytest.raises(AlreadyFinalized):
-            verifier.update(b"more data")
-
     def test_verify(self, backend):
         message = b"one little message"
         algorithm = hashes.SHA1()
@@ -469,27 +467,12 @@ class TestDSAVerification(object):
         with pytest.raises(ValueError):
             public_key.verify(b"\x00" * 128, digest, prehashed_alg)
 
-    def test_prehashed_unsupported_in_signer_ctx(self, backend):
-        private_key = DSA_KEY_1024.private_key(backend)
-        with pytest.raises(TypeError), pytest.warns(
-            CryptographyDeprecationWarning
-        ):
-            private_key.signer(
-                Prehashed(hashes.SHA1())  # type: ignore[arg-type]
-            )
 
-    def test_prehashed_unsupported_in_verifier_ctx(self, backend):
-        public_key = DSA_KEY_1024.private_key(backend).public_key()
-        with pytest.raises(TypeError), pytest.warns(
-            CryptographyDeprecationWarning
-        ):
-            public_key.verifier(
-                b"0" * 64, Prehashed(hashes.SHA1())  # type: ignore[arg-type]
-            )
-
-
-@pytest.mark.requires_backend_interface(interface=DSABackend)
-class TestDSASignature(object):
+@pytest.mark.supported(
+    only_if=lambda backend: backend.dsa_supported(),
+    skip_message="Does not support DSA.",
+)
+class TestDSASignature:
     def test_dsa_signing(self, backend, subtests):
         vectors = load_vectors_from_file(
             os.path.join("asymmetric", "DSA", "FIPS_186-3", "SigGen.txt"),
@@ -513,24 +496,13 @@ class TestDSASignature(object):
                     ),
                     x=vector["x"],
                 ).private_key(backend)
-                signature = private_key.sign(vector["msg"], algorithm())
+                signature = private_key.sign(vector["msg"], algorithm)
                 assert signature
 
                 private_key.public_key().verify(
-                    signature, vector["msg"], algorithm()
+                    signature, vector["msg"], algorithm
                 )
 
-    def test_use_after_finalize(self, backend):
-        private_key = DSA_KEY_1024.private_key(backend)
-        with pytest.warns(CryptographyDeprecationWarning):
-            signer = private_key.signer(hashes.SHA1())
-        signer.update(b"data")
-        signer.finalize()
-        with pytest.raises(AlreadyFinalized):
-            signer.finalize()
-        with pytest.raises(AlreadyFinalized):
-            signer.update(b"more data")
-
     def test_sign(self, backend):
         private_key = DSA_KEY_1024.private_key(backend)
         message = b"one little message"
@@ -561,7 +533,7 @@ class TestDSASignature(object):
             private_key.sign(digest, prehashed_alg)
 
 
-class TestDSANumbers(object):
+class TestDSANumbers:
     def test_dsa_parameter_numbers(self):
         parameter_numbers = dsa.DSAParameterNumbers(p=1, q=2, g=3)
         assert parameter_numbers.p == 1
@@ -641,7 +613,7 @@ class TestDSANumbers(object):
         )
 
 
-class TestDSANumberEquality(object):
+class TestDSANumberEquality:
     def test_parameter_numbers_eq(self):
         param = dsa.DSAParameterNumbers(1, 2, 3)
         assert param == dsa.DSAParameterNumbers(1, 2, 3)
@@ -693,9 +665,11 @@ class TestDSANumberEquality(object):
         assert priv != object()
 
 
-@pytest.mark.requires_backend_interface(interface=DSABackend)
-@pytest.mark.requires_backend_interface(interface=PEMSerializationBackend)
-class TestDSASerialization(object):
+@pytest.mark.supported(
+    only_if=lambda backend: backend.dsa_supported(),
+    skip_message="Does not support DSA.",
+)
+class TestDSASerialization:
     @pytest.mark.parametrize(
         ("fmt", "password"),
         itertools.product(
@@ -739,6 +713,10 @@ class TestDSASerialization(object):
             (serialization.Encoding.DER, serialization.PrivateFormat.Raw),
             (serialization.Encoding.Raw, serialization.PrivateFormat.Raw),
             (serialization.Encoding.X962, serialization.PrivateFormat.PKCS8),
+            (
+                serialization.Encoding.SMIME,
+                serialization.PrivateFormat.TraditionalOpenSSL,
+            ),
         ],
     )
     def test_private_bytes_rejects_invalid(self, encoding, fmt, backend):
@@ -868,7 +846,7 @@ class TestDSASerialization(object):
         )
         with pytest.raises(TypeError):
             key.private_bytes(
-                "notencoding",
+                "notencoding",  # type: ignore[arg-type]
                 serialization.PrivateFormat.PKCS8,
                 serialization.NoEncryption(),
             )
@@ -883,7 +861,7 @@ class TestDSASerialization(object):
         with pytest.raises(TypeError):
             key.private_bytes(
                 serialization.Encoding.PEM,
-                "invalidformat",
+                "invalidformat",  # type: ignore[arg-type]
                 serialization.NoEncryption(),
             )
 
@@ -898,7 +876,7 @@ class TestDSASerialization(object):
             key.private_bytes(
                 serialization.Encoding.PEM,
                 serialization.PrivateFormat.TraditionalOpenSSL,
-                "notanencalg",
+                "notanencalg",  # type: ignore[arg-type]
             )
 
     def test_private_bytes_unsupported_encryption_type(self, backend):
@@ -916,9 +894,11 @@ class TestDSASerialization(object):
             )
 
 
-@pytest.mark.requires_backend_interface(interface=DSABackend)
-@pytest.mark.requires_backend_interface(interface=PEMSerializationBackend)
-class TestDSAPEMPublicKeySerialization(object):
+@pytest.mark.supported(
+    only_if=lambda backend: backend.dsa_supported(),
+    skip_message="Does not support DSA.",
+)
+class TestDSAPEMPublicKeySerialization:
     @pytest.mark.parametrize(
         ("key_path", "loader_func", "encoding"),
         [
@@ -959,9 +939,11 @@ class TestDSAPEMPublicKeySerialization(object):
         )
         key = serialization.load_pem_public_key(key_bytes, backend)
 
-        ssh_bytes = key.public_bytes(
-            serialization.Encoding.OpenSSH, serialization.PublicFormat.OpenSSH
-        )
+        with pytest.warns(utils.DeprecatedIn40):
+            ssh_bytes = key.public_bytes(
+                serialization.Encoding.OpenSSH,
+                serialization.PublicFormat.OpenSSH,
+            )
         assert ssh_bytes == (
             b"ssh-dss AAAAB3NzaC1kc3MAAACBAKoJMMwUWCUiHK/6KKwolBlqJ4M95ewhJweR"
             b"aJQgd3Si57I4sNNvGySZosJYUIPrAUMpJEGNhn+qIS3RBx1NzrJ4J5StOTzAik1K"
