@@ -6,26 +6,20 @@
 import binascii
 import itertools
 import os
+import textwrap
 import typing
 from binascii import hexlify
 
 import pytest
 
-from cryptography import exceptions, utils, x509
-from cryptography.hazmat.backends.interfaces import (
-    EllipticCurveBackend,
-    PEMSerializationBackend,
-)
+from cryptography import exceptions, x509
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.utils import (
     Prehashed,
     encode_dss_signature,
 )
-from cryptography.utils import CryptographyDeprecationWarning
 
-from .fixtures_ec import EC_KEY_SECP384R1
-from .utils import skip_fips_traditional_openssl
 from ...doubles import DummyKeySerializationEncryption
 from ...utils import (
     load_fips_ecdsa_key_pair_vectors,
@@ -35,6 +29,8 @@ from ...utils import (
     load_vectors_from_file,
     raises_unsupported_algorithm,
 )
+from .fixtures_ec import EC_KEY_SECP384R1
+from .utils import skip_fips_traditional_openssl
 
 _HASH_TYPES: typing.Dict[str, typing.Type[hashes.HashAlgorithm]] = {
     "SHA-1": hashes.SHA1,
@@ -91,25 +87,21 @@ class DummySignatureAlgorithm(ec.EllipticCurveSignatureAlgorithm):
     algorithm = hashes.SHA256()
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_skip_curve_unsupported(backend):
     with pytest.raises(pytest.skip.Exception):
         _skip_curve_unsupported(backend, DummyCurve())
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_skip_exchange_algorithm_unsupported(backend):
     with pytest.raises(pytest.skip.Exception):
         _skip_exchange_algorithm_unsupported(backend, ec.ECDH(), DummyCurve())
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_skip_ecdsa_vector(backend):
     with pytest.raises(pytest.skip.Exception):
         _skip_ecdsa_vector(backend, DummyCurve, hashes.SHA256)
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_derive_private_key_success(backend):
     curve = ec.SECP256K1()
     _skip_curve_unsupported(backend, curve)
@@ -123,7 +115,6 @@ def test_derive_private_key_success(backend):
     assert private_numbers == derived_key.private_numbers()
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_derive_private_key_errors(backend):
     curve = ec.SECP256K1()
     _skip_curve_unsupported(backend, curve)
@@ -138,6 +129,15 @@ def test_derive_private_key_errors(backend):
         ec.derive_private_key(-7, curve, backend)
 
 
+def test_derive_point_at_infinity(backend):
+    curve = ec.SECP256R1()
+    _skip_curve_unsupported(backend, curve)
+    # order of the curve
+    q = 0xFFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551
+    with pytest.raises(ValueError, match="Unable to derive"):
+        ec.derive_private_key(q, ec.SECP256R1())
+
+
 def test_ec_numbers():
     numbers = ec.EllipticCurvePrivateNumbers(
         1, ec.EllipticCurvePublicNumbers(2, 3, DummyCurve())
@@ -170,73 +170,6 @@ def test_invalid_private_numbers_public_numbers():
         ec.EllipticCurvePrivateNumbers(1, None)  # type: ignore[arg-type]
 
 
-def test_encode_point():
-    # secp256r1 point
-    x = int(
-        "233ea3b0027127084cd2cd336a13aeef69c598d8af61369a36454a17c6c22aec", 16
-    )
-    y = int(
-        "3ea2c10a84153862be4ec82940f0543f9ba866af9751a6ee79d38460b35f442e", 16
-    )
-    pn = ec.EllipticCurvePublicNumbers(x, y, ec.SECP256R1())
-    with pytest.warns(utils.PersistentlyDeprecated2019):
-        data = pn.encode_point()
-    assert data == binascii.unhexlify(
-        "04233ea3b0027127084cd2cd336a13aeef69c598d8af61369a36454a17c6c22ae"
-        "c3ea2c10a84153862be4ec82940f0543f9ba866af9751a6ee79d38460b35f442e"
-    )
-
-
-def test_from_encoded_point():
-    # secp256r1 point
-    data = binascii.unhexlify(
-        "04233ea3b0027127084cd2cd336a13aeef69c598d8af61369a36454a17c6c22ae"
-        "c3ea2c10a84153862be4ec82940f0543f9ba866af9751a6ee79d38460b35f442e"
-    )
-    with pytest.warns(CryptographyDeprecationWarning):
-        pn = ec.EllipticCurvePublicNumbers.from_encoded_point(
-            ec.SECP256R1(), data
-        )
-    assert pn.x == int(
-        "233ea3b0027127084cd2cd336a13aeef69c598d8af61369a36454a17c6c22aec", 16
-    )
-    assert pn.y == int(
-        "3ea2c10a84153862be4ec82940f0543f9ba866af9751a6ee79d38460b35f442e", 16
-    )
-
-
-def test_from_encoded_point_invalid_length():
-    bad_data = binascii.unhexlify(
-        "04233ea3b0027127084cd2cd336a13aeef69c598d8af61369a36454a17c6c22ae"
-        "c3ea2c10a84153862be4ec82940f0543f9ba866af9751a6ee79d38460"
-    )
-    with pytest.raises(ValueError):
-        with pytest.warns(CryptographyDeprecationWarning):
-            ec.EllipticCurvePublicNumbers.from_encoded_point(
-                ec.SECP384R1(), bad_data
-            )
-
-
-def test_from_encoded_point_unsupported_point_no_backend():
-    # set to point type 2.
-    unsupported_type = binascii.unhexlify(
-        "02233ea3b0027127084cd2cd336a13aeef69c598d8af61369a36454a17c6c22a"
-    )
-    with pytest.raises(ValueError):
-        with pytest.warns(CryptographyDeprecationWarning):
-            ec.EllipticCurvePublicNumbers.from_encoded_point(
-                ec.SECP256R1(), unsupported_type
-            )
-
-
-def test_from_encoded_point_not_a_curve():
-    with pytest.raises(TypeError):
-        with pytest.warns(CryptographyDeprecationWarning):
-            ec.EllipticCurvePublicNumbers.from_encoded_point(
-                "notacurve", b"\x04data"  # type: ignore[arg-type]
-            )
-
-
 def test_ec_public_numbers_repr():
     pn = ec.EllipticCurvePublicNumbers(2, 3, ec.SECP256R1())
     assert repr(pn) == "<EllipticCurvePublicNumbers(curve=secp256r1, x=2, y=3>"
@@ -266,7 +199,6 @@ def test_ec_private_numbers_hash():
     assert hash(numbers1) != hash(numbers3)
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_ec_key_key_size(backend):
     curve = ec.SECP256R1()
     _skip_curve_unsupported(backend, curve)
@@ -275,8 +207,7 @@ def test_ec_key_key_size(backend):
     assert key.public_key().key_size == 256
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-class TestECWithNumbers(object):
+class TestECWithNumbers:
     def test_with_numbers(self, backend, subtests):
         vectors = itertools.product(
             load_vectors_from_file(
@@ -310,8 +241,7 @@ class TestECWithNumbers(object):
                 assert curve_type().name == priv_num.public_numbers.curve.name
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-class TestECDSAVectors(object):
+class TestECDSAVectors:
     def test_signing_with_example_keys(self, backend, subtests):
         vectors = itertools.product(
             load_vectors_from_file(
@@ -339,15 +269,13 @@ class TestECDSAVectors(object):
                 pkey = key.public_key()
                 assert pkey
 
-                with pytest.warns(CryptographyDeprecationWarning):
-                    signer = key.signer(ec.ECDSA(hash_type()))
-                signer.update(b"YELLOW SUBMARINE")
-                signature = signer.finalize()
+                signature = key.sign(
+                    b"YELLOW SUBMARINE", ec.ECDSA(hash_type())
+                )
 
-                with pytest.warns(CryptographyDeprecationWarning):
-                    verifier = pkey.verifier(signature, ec.ECDSA(hash_type()))
-                verifier.update(b"YELLOW SUBMARINE")
-                verifier.verify()
+                pkey.verify(
+                    signature, b"YELLOW SUBMARINE", ec.ECDSA(hash_type())
+                )
 
     @pytest.mark.parametrize("curve", ec._CURVE_TYPES.values())
     def test_generate_vector_curves(self, backend, curve):
@@ -381,21 +309,11 @@ class TestECDSAVectors(object):
 
         key = ec.generate_private_key(ec.SECP192R1(), backend)
 
-        with raises_unsupported_algorithm(
-            exceptions._Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ), pytest.warns(CryptographyDeprecationWarning):
-            key.signer(DummySignatureAlgorithm())
-
         with raises_unsupported_algorithm(
             exceptions._Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
         ):
             key.sign(b"somedata", DummySignatureAlgorithm())
 
-        with raises_unsupported_algorithm(
-            exceptions._Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ), pytest.warns(CryptographyDeprecationWarning):
-            key.public_key().verifier(b"", DummySignatureAlgorithm())
-
         with raises_unsupported_algorithm(
             exceptions._Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
         ):
@@ -487,6 +405,41 @@ class TestECDSAVectors(object):
         with pytest.raises(ValueError):
             numbers.public_key(backend)
 
+    def test_load_invalid_ec_key_from_pem(self, backend):
+        _skip_curve_unsupported(backend, ec.SECP256R1())
+
+        # BoringSSL rejects infinity points before it ever gets to us, so it
+        # uses a more generic error message.
+        match = (
+            r"infinity|invalid form"
+            if not backend._lib.CRYPTOGRAPHY_IS_BORINGSSL
+            else None
+        )
+        with pytest.raises(ValueError, match=match):
+            serialization.load_pem_public_key(
+                textwrap.dedent(
+                    """
+            -----BEGIN PUBLIC KEY-----
+            MBkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDAgAA
+            -----END PUBLIC KEY-----
+            """
+                ).encode(),
+                backend=backend,
+            )
+        with pytest.raises(ValueError, match=match):
+            serialization.load_pem_private_key(
+                textwrap.dedent(
+                    """
+            -----BEGIN PRIVATE KEY-----
+            MEECAQAwEwYHKoZIzj0CAQYIKoZIzj0DAQcEJzAlAgEBBCD/////AAAAAP//////
+            ////vOb6racXnoTzucrC/GMlUQ==
+            -----END PRIVATE KEY-----
+            """
+                ).encode(),
+                password=None,
+                backend=backend,
+            )
+
     def test_signatures(self, backend, subtests):
         vectors = itertools.chain(
             load_vectors_from_file(
@@ -611,25 +564,8 @@ class TestECDSAVectors(object):
                 b"\x00" * 32, data, ec.ECDSA(Prehashed(hashes.SHA256()))
             )
 
-    def test_prehashed_unsupported_in_signer_ctx(self, backend):
-        _skip_curve_unsupported(backend, ec.SECP256R1())
-        private_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        with pytest.raises(TypeError), pytest.warns(
-            CryptographyDeprecationWarning
-        ):
-            private_key.signer(ec.ECDSA(Prehashed(hashes.SHA1())))
 
-    def test_prehashed_unsupported_in_verifier_ctx(self, backend):
-        _skip_curve_unsupported(backend, ec.SECP256R1())
-        private_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        public_key = private_key.public_key()
-        with pytest.raises(TypeError), pytest.warns(
-            CryptographyDeprecationWarning
-        ):
-            public_key.verifier(b"0" * 64, ec.ECDSA(Prehashed(hashes.SHA1())))
-
-
-class TestECNumbersEquality(object):
+class TestECEquality:
     def test_public_numbers_eq(self):
         pub = ec.EllipticCurvePublicNumbers(1, 2, ec.SECP192R1())
         assert pub == ec.EllipticCurvePublicNumbers(1, 2, ec.SECP192R1())
@@ -665,10 +601,21 @@ class TestECNumbersEquality(object):
         )
         assert priv != object()
 
+    def test_public_key_equality(self, backend):
+        _skip_curve_unsupported(backend, ec.SECP256R1())
+        key_bytes = load_vectors_from_file(
+            os.path.join("asymmetric", "PKCS8", "ec_private_key.pem"),
+            lambda pemfile: pemfile.read().encode(),
+        )
+        key1 = serialization.load_pem_private_key(key_bytes, None).public_key()
+        key2 = serialization.load_pem_private_key(key_bytes, None).public_key()
+        key3 = ec.generate_private_key(ec.SECP256R1()).public_key()
+        assert key1 == key2
+        assert key1 != key3
+        assert key1 != object()
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-@pytest.mark.requires_backend_interface(interface=PEMSerializationBackend)
-class TestECSerialization(object):
+
+class TestECSerialization:
     @pytest.mark.parametrize(
         ("fmt", "password"),
         itertools.product(
@@ -857,7 +804,7 @@ class TestECSerialization(object):
         )
         with pytest.raises(TypeError):
             key.private_bytes(
-                "notencoding",
+                "notencoding",  # type: ignore[arg-type]
                 serialization.PrivateFormat.PKCS8,
                 serialization.NoEncryption(),
             )
@@ -873,7 +820,7 @@ class TestECSerialization(object):
         with pytest.raises(TypeError):
             key.private_bytes(
                 serialization.Encoding.PEM,
-                "invalidformat",
+                "invalidformat",  # type: ignore[arg-type]
                 serialization.NoEncryption(),
             )
 
@@ -889,7 +836,7 @@ class TestECSerialization(object):
             key.private_bytes(
                 serialization.Encoding.PEM,
                 serialization.PrivateFormat.TraditionalOpenSSL,
-                "notanencalg",
+                "notanencalg",  # type: ignore[arg-type]
             )
 
     def test_private_bytes_unsupported_encryption_type(self, backend):
@@ -924,9 +871,7 @@ class TestECSerialization(object):
         assert parsed_public
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-@pytest.mark.requires_backend_interface(interface=PEMSerializationBackend)
-class TestEllipticCurvePEMPublicKeySerialization(object):
+class TestEllipticCurvePEMPublicKeySerialization:
     @pytest.mark.parametrize(
         ("key_path", "loader_func", "encoding"),
         [
@@ -1001,7 +946,8 @@ class TestEllipticCurvePEMPublicKeySerialization(object):
         )
         with pytest.raises(TypeError):
             key.public_bytes(
-                "notencoding", serialization.PublicFormat.SubjectPublicKeyInfo
+                "notencoding",  # type: ignore[arg-type]
+                serialization.PublicFormat.SubjectPublicKeyInfo,
             )
 
     @pytest.mark.parametrize(
@@ -1046,7 +992,10 @@ class TestEllipticCurvePEMPublicKeySerialization(object):
             ),
         )
         with pytest.raises(TypeError):
-            key.public_bytes(serialization.Encoding.PEM, "invalidformat")
+            key.public_bytes(
+                serialization.Encoding.PEM,
+                "invalidformat",  # type: ignore[arg-type]
+            )
 
     def test_public_bytes_pkcs1_unsupported(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
@@ -1178,22 +1127,7 @@ class TestEllipticCurvePEMPublicKeySerialization(object):
         )
 
 
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-class TestECDSAVerification(object):
-    def test_signature_not_bytes(self, backend):
-        _skip_curve_unsupported(backend, ec.SECP256R1())
-        key = ec.generate_private_key(ec.SECP256R1(), backend)
-        public_key = key.public_key()
-        with pytest.raises(TypeError), pytest.warns(
-            CryptographyDeprecationWarning
-        ):
-            public_key.verifier(
-                1234, ec.ECDSA(hashes.SHA256())  # type: ignore[arg-type]
-            )
-
-
-@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-class TestECDH(object):
+class TestECDH:
     def test_key_exchange_with_vectors(self, backend, subtests):
         vectors = load_vectors_from_file(
             os.path.join(
@@ -1206,7 +1140,7 @@ class TestECDH(object):
         for vector in vectors:
             with subtests.test():
                 _skip_exchange_algorithm_unsupported(
-                    backend, ec.ECDH(), ec._CURVE_TYPES[vector["curve"]]
+                    backend, ec.ECDH(), ec._CURVE_TYPES[vector["curve"]]()
                 )
 
                 key_numbers = vector["IUT"]
@@ -1261,18 +1195,18 @@ class TestECDH(object):
         ),
     )
     def test_brainpool_kex(self, backend, vector):
-        curve = ec._CURVE_TYPES[vector["curve"].decode("ascii")]
+        curve = ec._CURVE_TYPES[vector["curve"].decode("ascii")]()
         _skip_exchange_algorithm_unsupported(backend, ec.ECDH(), curve)
         key = ec.EllipticCurvePrivateNumbers(
             int(vector["da"], 16),
             ec.EllipticCurvePublicNumbers(
-                int(vector["x_qa"], 16), int(vector["y_qa"], 16), curve()
+                int(vector["x_qa"], 16), int(vector["y_qa"], 16), curve
             ),
         ).private_key(backend)
         peer = ec.EllipticCurvePrivateNumbers(
             int(vector["db"], 16),
             ec.EllipticCurvePublicNumbers(
-                int(vector["x_qb"], 16), int(vector["y_qb"], 16), curve()
+                int(vector["x_qb"], 16), int(vector["y_qb"], 16), curve
             ),
         ).private_key(backend)
         shared_secret = key.exchange(ec.ECDH(), peer.public_key())
@@ -1289,11 +1223,12 @@ class TestECDH(object):
                 pemfile.read().encode(), None, backend
             ),
         )
+        assert isinstance(key, ec.EllipticCurvePrivateKey)
 
         with raises_unsupported_algorithm(
             exceptions._Reasons.UNSUPPORTED_EXCHANGE_ALGORITHM
         ):
-            key.exchange(None, key.public_key())
+            key.exchange(None, key.public_key())  # type: ignore[arg-type]
 
     def test_exchange_non_matching_curve(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
@@ -1305,6 +1240,7 @@ class TestECDH(object):
                 pemfile.read().encode(), None, backend
             ),
         )
+        assert isinstance(key, ec.EllipticCurvePrivateKey)
         public_key = EC_KEY_SECP384R1.public_numbers.public_key(backend)
 
         with pytest.raises(ValueError):
