@@ -4,22 +4,18 @@
 
 
 import base64
-import calendar
+import datetime
 import json
 import os
 import time
 
-import iso8601
-
 import pretend
-
 import pytest
 
+import cryptography_vectors
 from cryptography.fernet import Fernet, InvalidToken, MultiFernet
 from cryptography.hazmat.primitives.ciphers import algorithms, modes
 
-import cryptography_vectors
-
 
 def json_parametrize(keys, filename):
     vector_file = cryptography_vectors.open_vector_file(
@@ -49,7 +45,7 @@ class TestFernet:
         f = Fernet(secret.encode("ascii"), backend=backend)
         actual_token = f._encrypt_from_parts(
             src.encode("ascii"),
-            calendar.timegm(iso8601.parse_date(now).utctimetuple()),
+            int(datetime.datetime.fromisoformat(now).timestamp()),
             bytes(iv),
         )
         assert actual_token == token.encode("ascii")
@@ -63,7 +59,7 @@ class TestFernet:
     ):
         # secret & token are both str
         f = Fernet(secret.encode("ascii"), backend=backend)
-        current_time = calendar.timegm(iso8601.parse_date(now).utctimetuple())
+        current_time = int(datetime.datetime.fromisoformat(now).timestamp())
         payload = f.decrypt_at_time(
             token,  # str
             ttl=ttl_sec,
@@ -89,7 +85,7 @@ class TestFernet:
     @json_parametrize(("secret", "token", "now", "ttl_sec"), "invalid.json")
     def test_invalid(self, secret, token, now, ttl_sec, backend, monkeypatch):
         f = Fernet(secret.encode("ascii"), backend=backend)
-        current_time = calendar.timegm(iso8601.parse_date(now).utctimetuple())
+        current_time = int(datetime.datetime.fromisoformat(now).timestamp())
         with pytest.raises(InvalidToken):
             f.decrypt_at_time(
                 token.encode("ascii"),
