@@ -19,10 +19,7 @@ class TestFixedPool:
             events.append(("create", c))
             return c
 
-        def destroy(c):
-            events.append(("destroy", c))
-
-        pool = FixedPool(create, destroy)
+        pool = FixedPool(create)
         assert events == [("create", 1)]
         with pool.acquire() as c:
             assert c == 1
@@ -32,9 +29,9 @@ class TestFixedPool:
                 assert c == 2
                 assert events == [("create", 1), ("create", 2)]
 
-            assert events == [("create", 1), ("create", 2), ("destroy", 2)]
+            assert events == [("create", 1), ("create", 2)]
 
-        assert events == [("create", 1), ("create", 2), ("destroy", 2)]
+        assert events == [("create", 1), ("create", 2)]
 
         del pool
         gc.collect()
@@ -44,18 +41,13 @@ class TestFixedPool:
         assert events == [
             ("create", 1),
             ("create", 2),
-            ("destroy", 2),
-            ("destroy", 1),
         ]
 
     def test_thread_stress(self):
         def create():
             return None
 
-        def destroy(c):
-            pass
-
-        pool = FixedPool(create, destroy)
+        pool = FixedPool(create)
 
         def thread_fn():
             with pool.acquire():
