@@ -13,10 +13,8 @@ from contextlib import contextmanager
 
 import pytest
 
-from cryptography.exceptions import UnsupportedAlgorithm
-
 import cryptography_vectors
-
+from cryptography.exceptions import UnsupportedAlgorithm
 
 HashVector = collections.namedtuple("HashVector", ["message", "digest"])
 KeyedHashVector = collections.namedtuple(
@@ -35,7 +33,7 @@ def raises_unsupported_algorithm(reason):
     with pytest.raises(UnsupportedAlgorithm) as exc_info:
         yield exc_info
 
-    assert exc_info.value._reason is reason
+    assert exc_info.value._reason == reason
 
 
 T = typing.TypeVar("T")
@@ -68,7 +66,7 @@ def load_nist_vectors(vector_data):
             continue
 
         # Build our data using a simple Key = Value format
-        name, value = [c.strip() for c in line.split("=")]
+        name, value = (c.strip() for c in line.split("="))
 
         # Some tests (PBKDF2) contain \0, which should be interpreted as a
         # null character rather than literal.
@@ -109,7 +107,7 @@ def load_cryptrec_vectors(vector_data):
                 {"key": key, "plaintext": pt, "ciphertext": ct}
             )
         else:
-            raise ValueError("Invalid line in file '{}'".format(line))
+            raise ValueError(f"Invalid line in file '{line}'")
     return cryptrec_list
 
 
@@ -245,9 +243,6 @@ def load_pkcs1_vectors(vector_data):
             attr = None
 
         if private_key_vector is None or public_key_vector is None:
-            # Random garbage to defeat CPython's peephole optimizer so that
-            # coverage records correctly: https://bugs.python.org/issue2506
-            1 + 1
             continue
 
         if line.startswith("# Private key"):
@@ -302,7 +297,7 @@ def load_rsa_nist_vectors(vector_data):
             continue
 
         # Build our data using a simple Key = Value format
-        name, value = [c.strip() for c in line.split("=")]
+        name, value = (c.strip() for c in line.split("="))
 
         if name == "n":
             n = int(value, 16)
@@ -398,7 +393,7 @@ def load_fips_dsa_sig_vectors(vector_data):
         if line.startswith("[mod"):
             continue
 
-        name, value = [c.strip() for c in line.split("=")]
+        name, value = (c.strip() for c in line.split("="))
 
         if name == "P":
             vectors.append(
@@ -773,7 +768,7 @@ def load_nist_kbkdf_vectors(vector_data):
 
         if line.startswith("[") and line.endswith("]"):
             tag_data = line[1:-1]
-            name, value = [c.strip() for c in tag_data.split("=")]
+            name, value = (c.strip() for c in tag_data.split("="))
             if value.endswith("_BITS"):
                 value = int(value.split("_")[0])
                 tag.update({name.lower(): value})
@@ -785,10 +780,10 @@ def load_nist_kbkdf_vectors(vector_data):
             test_data.update(tag)
             vectors.append(test_data)
         elif line.startswith(("L", "DataBeforeCtrLen", "DataAfterCtrLen")):
-            name, value = [c.strip() for c in line.split("=")]
+            name, value = (c.strip() for c in line.split("="))
             test_data[name.lower()] = int(value)
         else:
-            name, value = [c.strip() for c in line.split("=")]
+            name, value = (c.strip() for c in line.split("="))
             test_data[name.lower()] = value.encode("ascii")
 
     return vectors
@@ -830,7 +825,7 @@ def load_nist_ccm_vectors(vector_data):
         # Some of the CCM vectors have global values for this. They are always
         # at the top before the first section header (see: VADT, VNT, VPT)
         if line.startswith(("Alen", "Plen", "Nlen", "Tlen")):
-            name, value = [c.strip() for c in line.split("=")]
+            name, value = (c.strip() for c in line.split("="))
             global_data[name.lower()] = int(value)
             continue
 
@@ -841,11 +836,11 @@ def load_nist_ccm_vectors(vector_data):
             section = line[1:-1]
             items = [c.strip() for c in section.split(",")]
             for item in items:
-                name, value = [c.strip() for c in item.split("=")]
+                name, value = (c.strip() for c in item.split("="))
                 section_data[name.lower()] = int(value)
             continue
 
-        name, value = [c.strip() for c in line.split("=")]
+        name, value = (c.strip() for c in line.split("="))
 
         if name.lower() in ("key", "nonce") and new_section:
             section_data[name.lower()] = value.encode("ascii")
@@ -901,20 +896,27 @@ class WycheproofTest:
         )
 
     @property
-    def valid(self):
+    def valid(self) -> bool:
         return self.testcase["result"] == "valid"
 
     @property
-    def acceptable(self):
+    def acceptable(self) -> bool:
         return self.testcase["result"] == "acceptable"
 
     @property
-    def invalid(self):
+    def invalid(self) -> bool:
         return self.testcase["result"] == "invalid"
 
-    def has_flag(self, flag):
+    def has_flag(self, flag: str) -> bool:
         return flag in self.testcase["flags"]
 
+    def cache_value_to_group(self, cache_key: str, func):
+        cache_val = self.testgroup.get(cache_key)
+        if cache_val is not None:
+            return cache_val
+        self.testgroup[cache_key] = cache_val = func()
+        return cache_val
+
 
 def load_wycheproof_tests(wycheproof, test_file):
     path = os.path.join(wycheproof, "testvectors", test_file)
