@@ -8,16 +8,14 @@ import binascii
 import pytest
 
 from cryptography.exceptions import InvalidTag
-from cryptography.hazmat.backends.interfaces import CipherBackend
 from cryptography.hazmat.primitives import padding
 from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
 from cryptography.hazmat.primitives.ciphers.aead import AESCCM, AESGCM
 
-from .utils import wycheproof_tests
 from ..hazmat.primitives.test_aead import _aead_supported
+from .utils import wycheproof_tests
 
 
-@pytest.mark.requires_backend_interface(interface=CipherBackend)
 @wycheproof_tests("aes_cbc_pkcs5_test.json")
 def test_aes_cbc_pkcs5(backend, wycheproof):
     key = binascii.unhexlify(wycheproof.testcase["key"])
@@ -45,7 +43,6 @@ def test_aes_cbc_pkcs5(backend, wycheproof):
             unpadder.update(padded_msg) + unpadder.finalize()
 
 
-@pytest.mark.requires_backend_interface(interface=CipherBackend)
 @wycheproof_tests("aes_gcm_test.json")
 def test_aes_gcm(backend, wycheproof):
     key = binascii.unhexlify(wycheproof.testcase["key"])
@@ -90,7 +87,6 @@ def test_aes_gcm(backend, wycheproof):
             dec.finalize()
 
 
-@pytest.mark.requires_backend_interface(interface=CipherBackend)
 @wycheproof_tests("aes_gcm_test.json")
 def test_aes_gcm_aead_api(backend, wycheproof):
     key = binascii.unhexlify(wycheproof.testcase["key"])
@@ -124,7 +120,6 @@ def test_aes_gcm_aead_api(backend, wycheproof):
     not _aead_supported(AESCCM),
     reason="Requires OpenSSL with AES-CCM support",
 )
-@pytest.mark.requires_backend_interface(interface=CipherBackend)
 @wycheproof_tests("aes_ccm_test.json")
 def test_aes_ccm_aead_api(backend, wycheproof):
     key = binascii.unhexlify(wycheproof.testcase["key"])
