@@ -13,7 +13,6 @@ from cryptography.hazmat.primitives.asymmetric import padding, rsa
 
 from .utils import wycheproof_tests
 
-
 _DIGESTS = {
     "SHA-1": hashes.SHA1(),
     "SHA-224": hashes.SHA224(),
@@ -64,8 +63,11 @@ def should_verify(backend, wycheproof):
     "rsa_signature_4096_sha512_256_test.json",
 )
 def test_rsa_pkcs1v15_signature(backend, wycheproof):
-    key = serialization.load_der_public_key(
-        binascii.unhexlify(wycheproof.testgroup["keyDer"]), backend
+    key = wycheproof.cache_value_to_group(
+        "cached_key",
+        lambda: serialization.load_der_public_key(
+            binascii.unhexlify(wycheproof.testgroup["keyDer"]),
+        ),
     )
     assert isinstance(key, rsa.RSAPublicKey)
     digest = _DIGESTS[wycheproof.testgroup["sha"]]
@@ -94,16 +96,22 @@ def test_rsa_pkcs1v15_signature(backend, wycheproof):
 
 @wycheproof_tests("rsa_sig_gen_misc_test.json")
 def test_rsa_pkcs1v15_signature_generation(backend, wycheproof):
-    key = serialization.load_pem_private_key(
-        wycheproof.testgroup["privateKeyPem"].encode(),
-        password=None,
-        backend=backend,
+    key = wycheproof.cache_value_to_group(
+        "cached_key",
+        lambda: serialization.load_pem_private_key(
+            wycheproof.testgroup["privateKeyPem"].encode("ascii"),
+            password=None,
+            unsafe_skip_rsa_key_validation=True,
+        ),
     )
     assert isinstance(key, rsa.RSAPrivateKey)
+
     digest = _DIGESTS[wycheproof.testgroup["sha"]]
     assert digest is not None
     if backend._fips_enabled:
-        if key.key_size < 2048 or isinstance(digest, hashes.SHA1):
+        if key.key_size < backend._fips_rsa_min_key_size or isinstance(
+            digest, hashes.SHA1
+        ):
             pytest.skip(
                 "Invalid params for FIPS. key: {} bits, digest: {}".format(
                     key.key_size, digest.name
@@ -130,11 +138,17 @@ def test_rsa_pkcs1v15_signature_generation(backend, wycheproof):
     "rsa_pss_misc_test.json",
 )
 def test_rsa_pss_signature(backend, wycheproof):
-    key = serialization.load_der_public_key(
-        binascii.unhexlify(wycheproof.testgroup["keyDer"]), backend
+    digest = _DIGESTS[wycheproof.testgroup["sha"]]
+    if backend._fips_enabled and isinstance(digest, hashes.SHA1):
+        pytest.skip("Invalid params for FIPS. SHA1 is disallowed")
+
+    key = wycheproof.cache_value_to_group(
+        "cached_key",
+        lambda: serialization.load_der_public_key(
+            binascii.unhexlify(wycheproof.testgroup["keyDer"]),
+        ),
     )
     assert isinstance(key, rsa.RSAPublicKey)
-    digest = _DIGESTS[wycheproof.testgroup["sha"]]
     mgf_digest = _DIGESTS[wycheproof.testgroup["mgfSha"]]
 
     if digest is None or mgf_digest is None:
@@ -189,22 +203,32 @@ def test_rsa_pss_signature(backend, wycheproof):
     "rsa_oaep_misc_test.json",
 )
 def test_rsa_oaep_encryption(backend, wycheproof):
-    key = serialization.load_pem_private_key(
-        wycheproof.testgroup["privateKeyPem"].encode("ascii"),
-        password=None,
-        backend=backend,
-    )
-    assert isinstance(key, rsa.RSAPrivateKey)
     digest = _DIGESTS[wycheproof.testgroup["sha"]]
     mgf_digest = _DIGESTS[wycheproof.testgroup["mgfSha"]]
     assert digest is not None
     assert mgf_digest is not None
-
     padding_algo = padding.OAEP(
         mgf=padding.MGF1(algorithm=mgf_digest),
         algorithm=digest,
         label=binascii.unhexlify(wycheproof.testcase["label"]),
     )
+    if not backend.rsa_encryption_supported(padding_algo):
+        pytest.skip(
+            f"Does not support OAEP using {mgf_digest.name} MGF1 "
+            f"or {digest.name} hash."
+        )
+
+    key = wycheproof.cache_value_to_group(
+        "cached_key",
+        lambda: serialization.load_pem_private_key(
+            wycheproof.testgroup["privateKeyPem"].encode("ascii"),
+            password=None,
+            unsafe_skip_rsa_key_validation=True,
+        ),
+    )
+    assert isinstance(key, rsa.RSAPrivateKey)
+    if backend._fips_enabled and key.key_size < backend._fips_rsa_min_key_size:
+        pytest.skip("Invalid params for FIPS. <2048 bit keys are disallowed")
 
     if wycheproof.valid or wycheproof.acceptable:
         pt = key.decrypt(
@@ -218,16 +242,25 @@ def test_rsa_oaep_encryption(backend, wycheproof):
             )
 
 
+@pytest.mark.supported(
+    only_if=lambda backend: backend.rsa_encryption_supported(
+        padding.PKCS1v15()
+    ),
+    skip_message="Does not support PKCS1v1.5 for encryption.",
+)
 @wycheproof_tests(
     "rsa_pkcs1_2048_test.json",
     "rsa_pkcs1_3072_test.json",
     "rsa_pkcs1_4096_test.json",
 )
 def test_rsa_pkcs1_encryption(backend, wycheproof):
-    key = serialization.load_pem_private_key(
-        wycheproof.testgroup["privateKeyPem"].encode("ascii"),
-        password=None,
-        backend=backend,
+    key = wycheproof.cache_value_to_group(
+        "cached_key",
+        lambda: serialization.load_pem_private_key(
+            wycheproof.testgroup["privateKeyPem"].encode("ascii"),
+            password=None,
+            unsafe_skip_rsa_key_validation=True,
+        ),
     )
     assert isinstance(key, rsa.RSAPrivateKey)
 
@@ -237,8 +270,18 @@ def test_rsa_pkcs1_encryption(backend, wycheproof):
         )
         assert pt == binascii.unhexlify(wycheproof.testcase["msg"])
     else:
-        with pytest.raises(ValueError):
-            key.decrypt(
-                binascii.unhexlify(wycheproof.testcase["ct"]),
-                padding.PKCS1v15(),
-            )
+        if backend._lib.Cryptography_HAS_IMPLICIT_RSA_REJECTION:
+            try:
+                assert key.decrypt(
+                    binascii.unhexlify(wycheproof.testcase["ct"]),
+                    padding.PKCS1v15(),
+                ) != binascii.unhexlify(wycheproof.testcase["ct"])
+            except ValueError:
+                # Some raise ValueError due to length mismatch.
+                pass
+        else:
+            with pytest.raises(ValueError):
+                key.decrypt(
+                    binascii.unhexlify(wycheproof.testcase["ct"]),
+                    padding.PKCS1v15(),
+                )
