@@ -12,13 +12,13 @@ import pytest
 from cryptography import x509
 from cryptography.exceptions import UnsupportedAlgorithm
 from cryptography.hazmat.primitives import hashes, serialization
-from cryptography.hazmat.primitives.asymmetric import ec, ed25519, ed448
+from cryptography.hazmat.primitives.asymmetric import ec, ed448, ed25519, rsa
 from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
 from cryptography.x509 import ocsp
 
-from .test_x509 import _load_cert
 from ..hazmat.primitives.fixtures_ec import EC_KEY_SECP256R1
-from ..utils import load_vectors_from_file
+from ..utils import load_vectors_from_file, raises_unsupported_algorithm
+from .test_x509 import DummyExtension, _load_cert
 
 
 def _load_data(filename, loader):
@@ -28,17 +28,13 @@ def _load_data(filename, loader):
 
 
 def _cert_and_issuer():
-    from cryptography.hazmat.backends.openssl.backend import backend
-
     cert = _load_cert(
         os.path.join("x509", "cryptography.io.pem"),
         x509.load_pem_x509_certificate,
-        backend,
     )
     issuer = _load_cert(
         os.path.join("x509", "rapidssl_sha256_ca_g3.pem"),
         x509.load_pem_x509_certificate,
-        backend,
     )
     return cert, issuer
 
@@ -72,7 +68,7 @@ def _generate_root(private_key=None, algorithm=hashes.SHA256()):
     return cert, private_key
 
 
-class TestOCSPRequest(object):
+class TestOCSPRequest:
     def test_bad_request(self):
         with pytest.raises(ValueError):
             ocsp.load_der_ocsp_request(b"invalid")
@@ -103,9 +99,42 @@ class TestOCSPRequest(object):
         ext = req.extensions[0]
         assert ext.critical is False
         assert ext.value == x509.OCSPNonce(
-            b"\x04\x10{\x80Z\x1d7&\xb8\xb8OH\xd2\xf8\xbf\xd7-\xfd"
+            b"{\x80Z\x1d7&\xb8\xb8OH\xd2\xf8\xbf\xd7-\xfd"
         )
 
+    def test_load_request_with_acceptable_responses(self):
+        req = _load_data(
+            os.path.join("x509", "ocsp", "req-acceptable-responses.der"),
+            ocsp.load_der_ocsp_request,
+        )
+        assert len(req.extensions) == 1
+        ext = req.extensions[0]
+        assert ext.critical is False
+        assert ext.value == x509.OCSPAcceptableResponses(
+            [x509.ObjectIdentifier("1.3.6.1.5.5.7.48.1.1")]
+        )
+
+    def test_load_request_with_unknown_extension(self):
+        req = _load_data(
+            os.path.join("x509", "ocsp", "req-ext-unknown-oid.der"),
+            ocsp.load_der_ocsp_request,
+        )
+        assert len(req.extensions) == 1
+        ext = req.extensions[0]
+        assert ext.critical is False
+        assert ext.value == x509.UnrecognizedExtension(
+            x509.ObjectIdentifier("1.3.6.1.5.5.7.48.1.2213"),
+            b"\x04\x10{\x80Z\x1d7&\xb8\xb8OH\xd2\xf8\xbf\xd7-\xfd",
+        )
+
+    def test_load_request_with_duplicate_extension(self):
+        req = _load_data(
+            os.path.join("x509", "ocsp", "req-duplicate-ext.der"),
+            ocsp.load_der_ocsp_request,
+        )
+        with pytest.raises(x509.DuplicateExtension):
+            req.extensions
+
     def test_load_request_two_requests(self):
         with pytest.raises(NotImplementedError):
             _load_data(
@@ -118,7 +147,7 @@ class TestOCSPRequest(object):
             os.path.join("x509", "ocsp", "req-invalid-hash-alg.der"),
             ocsp.load_der_ocsp_request,
         )
-        with pytest.raises(UnsupportedAlgorithm):
+        with raises_unsupported_algorithm(None):
             req.hash_algorithm
 
     def test_serialize_request(self):
@@ -141,13 +170,56 @@ class TestOCSPRequest(object):
             req.public_bytes(serialization.Encoding.PEM)
 
 
-class TestOCSPRequestBuilder(object):
-    def test_add_two_certs(self):
+class TestOCSPRequestBuilder:
+    def test_add_cert_twice(self):
         cert, issuer = _cert_and_issuer()
         builder = ocsp.OCSPRequestBuilder()
         builder = builder.add_certificate(cert, issuer, hashes.SHA1())
+        # Fails calling a second time
         with pytest.raises(ValueError):
             builder.add_certificate(cert, issuer, hashes.SHA1())
+        # Fails calling a second time with add_certificate_by_hash
+        with pytest.raises(ValueError):
+            builder.add_certificate_by_hash(
+                b"0" * 20, b"0" * 20, 1, hashes.SHA1()
+            )
+
+    def test_add_cert_by_hash_twice(self):
+        cert, issuer = _cert_and_issuer()
+        builder = ocsp.OCSPRequestBuilder()
+        builder = builder.add_certificate_by_hash(
+            b"0" * 20, b"0" * 20, 1, hashes.SHA1()
+        )
+        # Fails calling a second time
+        with pytest.raises(ValueError):
+            builder.add_certificate_by_hash(
+                b"0" * 20, b"0" * 20, 1, hashes.SHA1()
+            )
+        # Fails calling a second time with add_certificate
+        with pytest.raises(ValueError):
+            builder.add_certificate(cert, issuer, hashes.SHA1())
+
+    def test_add_cert_by_hash_bad_hash(self):
+        builder = ocsp.OCSPRequestBuilder()
+        with pytest.raises(ValueError):
+            builder.add_certificate_by_hash(
+                b"0" * 20, b"0" * 20, 1, "notahash"  # type:ignore[arg-type]
+            )
+        with pytest.raises(ValueError):
+            builder.add_certificate_by_hash(
+                b"0" * 19, b"0" * 20, 1, hashes.SHA1()
+            )
+        with pytest.raises(ValueError):
+            builder.add_certificate_by_hash(
+                b"0" * 20, b"0" * 21, 1, hashes.SHA1()
+            )
+        with pytest.raises(TypeError):
+            builder.add_certificate_by_hash(
+                b"0" * 20,
+                b"0" * 20,
+                "notanint",  # type:ignore[arg-type]
+                hashes.SHA1(),
+            )
 
     def test_create_ocsp_request_no_req(self):
         builder = ocsp.OCSPRequestBuilder()
@@ -174,6 +246,16 @@ class TestOCSPRequestBuilder(object):
                 False,
             )
 
+    def test_unsupported_extension(self):
+        cert, issuer = _cert_and_issuer()
+        builder = (
+            ocsp.OCSPRequestBuilder()
+            .add_extension(DummyExtension(), critical=False)
+            .add_certificate(cert, issuer, hashes.SHA256())
+        )
+        with pytest.raises(NotImplementedError):
+            builder.build()
+
     def test_create_ocsp_request_invalid_cert(self):
         cert, issuer = _cert_and_issuer()
         builder = ocsp.OCSPRequestBuilder()
@@ -221,8 +303,30 @@ class TestOCSPRequestBuilder(object):
         assert req.extensions[0].oid == ext.oid
         assert req.extensions[0].critical is critical
 
+    def test_add_cert_by_hash(self):
+        cert, issuer = _cert_and_issuer()
+        builder = ocsp.OCSPRequestBuilder()
+        h = hashes.Hash(hashes.SHA1())
+        h.update(cert.issuer.public_bytes())
+        issuer_name_hash = h.finalize()
+        # issuer_key_hash is a hash of the public key BitString DER,
+        # not the subjectPublicKeyInfo
+        issuer_key_hash = base64.b64decode(b"w5zz/NNGCDS7zkZ/oHxb8+IIy1k=")
+        builder = builder.add_certificate_by_hash(
+            issuer_name_hash,
+            issuer_key_hash,
+            cert.serial_number,
+            hashes.SHA1(),
+        )
+        req = builder.build()
+        serialized = req.public_bytes(serialization.Encoding.DER)
+        assert serialized == base64.b64decode(
+            b"MEMwQTA/MD0wOzAJBgUrDgMCGgUABBRAC0Z68eay0wmDug1gfn5ZN0gkxAQUw5zz"
+            b"/NNGCDS7zkZ/oHxb8+IIy1kCAj8g"
+        )
 
-class TestOCSPResponseBuilder(object):
+
+class TestOCSPResponseBuilder:
     def test_add_response_twice(self):
         cert, issuer = _cert_and_issuer()
         time = datetime.datetime.now()
@@ -413,6 +517,31 @@ class TestOCSPResponseBuilder(object):
                 "notanextension", True  # type: ignore[arg-type]
             )
 
+    def test_unsupported_extension(self):
+        root_cert, private_key = _generate_root()
+        cert, issuer = _cert_and_issuer()
+        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        this_update = current_time - datetime.timedelta(days=1)
+        next_update = this_update + datetime.timedelta(days=7)
+
+        builder = (
+            ocsp.OCSPResponseBuilder()
+            .responder_id(ocsp.OCSPResponderEncoding.NAME, root_cert)
+            .add_response(
+                cert,
+                issuer,
+                hashes.SHA256(),
+                ocsp.OCSPCertStatus.GOOD,
+                this_update,
+                next_update,
+                None,
+                None,
+            )
+            .add_extension(DummyExtension(), critical=False)
+        )
+        with pytest.raises(NotImplementedError):
+            builder.sign(private_key, hashes.SHA256())
+
     def test_sign_no_response(self):
         builder = ocsp.OCSPResponseBuilder()
         root_cert, private_key = _generate_root()
@@ -530,6 +659,33 @@ class TestOCSPResponseBuilder(object):
             resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA256())
         )
 
+    def test_sign_unknown_cert(self):
+        builder = ocsp.OCSPResponseBuilder()
+        cert, issuer = _cert_and_issuer()
+        root_cert, private_key = _generate_root()
+        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        this_update = current_time - datetime.timedelta(days=1)
+        next_update = this_update + datetime.timedelta(days=7)
+        builder = builder.responder_id(
+            ocsp.OCSPResponderEncoding.NAME, root_cert
+        ).add_response(
+            cert,
+            issuer,
+            hashes.SHA1(),
+            ocsp.OCSPCertStatus.UNKNOWN,
+            this_update,
+            next_update,
+            None,
+            None,
+        )
+        resp = builder.sign(private_key, hashes.SHA384())
+        assert resp.certificate_status == ocsp.OCSPCertStatus.UNKNOWN
+        assert resp.this_update == this_update
+        assert resp.next_update == next_update
+        private_key.public_key().verify(
+            resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA384())
+        )
+
     def test_sign_with_appended_certs(self):
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
@@ -722,8 +878,81 @@ class TestOCSPResponseBuilder(object):
                 ocsp.OCSPResponseStatus.SUCCESSFUL
             )
 
+    def test_sign_unknown_private_key(self, backend):
+        builder = ocsp.OCSPResponseBuilder()
+        cert, issuer = _cert_and_issuer()
+        root_cert, _ = _generate_root()
+        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        this_update = current_time - datetime.timedelta(days=1)
+        next_update = this_update + datetime.timedelta(days=7)
+        builder = builder.responder_id(
+            ocsp.OCSPResponderEncoding.NAME, root_cert
+        ).add_response(
+            cert,
+            issuer,
+            hashes.SHA1(),
+            ocsp.OCSPCertStatus.GOOD,
+            this_update,
+            next_update,
+            None,
+            None,
+        )
+        with pytest.raises(TypeError):
+            builder.sign(object(), hashes.SHA256())  # type:ignore[arg-type]
 
-class TestSignedCertificateTimestampsExtension(object):
+    @pytest.mark.supported(
+        only_if=lambda backend: backend.hash_supported(
+            hashes.BLAKE2b(digest_size=64)
+        ),
+        skip_message="Does not support BLAKE2b",
+    )
+    def test_sign_unrecognized_hash_algorithm(self, backend):
+        builder = ocsp.OCSPResponseBuilder()
+        cert, issuer = _cert_and_issuer()
+        root_cert, private_key = _generate_root()
+        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        this_update = current_time - datetime.timedelta(days=1)
+        next_update = this_update + datetime.timedelta(days=7)
+        builder = builder.responder_id(
+            ocsp.OCSPResponderEncoding.NAME, root_cert
+        ).add_response(
+            cert,
+            issuer,
+            hashes.SHA1(),
+            ocsp.OCSPCertStatus.GOOD,
+            this_update,
+            next_update,
+            None,
+            None,
+        )
+
+        with pytest.raises(UnsupportedAlgorithm):
+            builder.sign(private_key, hashes.BLAKE2b(digest_size=64))
+
+    def test_sign_none_hash_not_eddsa(self):
+        builder = ocsp.OCSPResponseBuilder()
+        cert, issuer = _cert_and_issuer()
+        root_cert, private_key = _generate_root()
+        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        this_update = current_time - datetime.timedelta(days=1)
+        next_update = this_update + datetime.timedelta(days=7)
+        builder = builder.responder_id(
+            ocsp.OCSPResponderEncoding.NAME, root_cert
+        ).add_response(
+            cert,
+            issuer,
+            hashes.SHA1(),
+            ocsp.OCSPCertStatus.GOOD,
+            this_update,
+            next_update,
+            None,
+            None,
+        )
+        with pytest.raises(TypeError):
+            builder.sign(private_key, None)
+
+
+class TestSignedCertificateTimestampsExtension:
     def test_init(self):
         with pytest.raises(TypeError):
             x509.SignedCertificateTimestamps(
@@ -735,10 +964,6 @@ class TestSignedCertificateTimestampsExtension(object):
             "<SignedCertificateTimestamps([])>"
         )
 
-    @pytest.mark.supported(
-        only_if=lambda backend: (backend._lib.Cryptography_HAS_SCT),
-        skip_message="Requires CT support",
-    )
     def test_eq(self, backend):
         sct1 = (
             _load_data(
@@ -762,10 +987,6 @@ class TestSignedCertificateTimestampsExtension(object):
         )
         assert sct1 == sct2
 
-    @pytest.mark.supported(
-        only_if=lambda backend: (backend._lib.Cryptography_HAS_SCT),
-        skip_message="Requires CT support",
-    )
     def test_ne(self, backend):
         sct1 = (
             _load_data(
@@ -781,10 +1002,6 @@ class TestSignedCertificateTimestampsExtension(object):
         assert sct1 != sct2
         assert sct1 != object()
 
-    @pytest.mark.supported(
-        only_if=lambda backend: (backend._lib.Cryptography_HAS_SCT),
-        skip_message="Requires CT support",
-    )
     def test_hash(self, backend):
         sct1 = (
             _load_data(
@@ -810,8 +1027,24 @@ class TestSignedCertificateTimestampsExtension(object):
         assert hash(sct1) == hash(sct2)
         assert hash(sct1) != hash(sct3)
 
+    def test_entry_type(self, backend):
+        [sct, _, _, _] = (
+            _load_data(
+                os.path.join("x509", "ocsp", "resp-sct-extension.der"),
+                ocsp.load_der_ocsp_response,
+            )
+            .single_extensions.get_extension_for_class(
+                x509.SignedCertificateTimestamps
+            )
+            .value
+        )
+        assert (
+            sct.entry_type
+            == x509.certificate_transparency.LogEntryType.X509_CERTIFICATE
+        )
 
-class TestOCSPResponse(object):
+
+class TestOCSPResponse:
     def test_bad_response(self):
         with pytest.raises(ValueError):
             ocsp.load_der_ocsp_response(b"invalid")
@@ -821,12 +1054,9 @@ class TestOCSPResponse(object):
             os.path.join("x509", "ocsp", "resp-sha256.der"),
             ocsp.load_der_ocsp_response,
         )
-        from cryptography.hazmat.backends.openssl.backend import backend
-
         issuer = _load_cert(
             os.path.join("x509", "letsencryptx3.pem"),
             x509.load_pem_x509_certificate,
-            backend,
         )
         assert resp.response_status == ocsp.OCSPResponseStatus.SUCCESSFUL
         assert (
@@ -849,7 +1079,9 @@ class TestOCSPResponse(object):
             b"mMEfd265tE5t6ZFZe/zqOyhAhIDHHh6fckClQB7xfIiCztSevCAABgPMjAxODA4"
             b"MzAxMTAwMDBaoBEYDzIwMTgwOTA2MTEwMDAwWg=="
         )
-        issuer.public_key().verify(
+        public_key = issuer.public_key()
+        assert isinstance(public_key, rsa.RSAPublicKey)
+        public_key.verify(
             resp.signature,
             resp.tbs_response_bytes,
             PKCS1v15(),
@@ -875,11 +1107,62 @@ class TestOCSPResponse(object):
         assert len(resp.extensions) == 0
 
     def test_load_multi_valued_response(self):
+        resp = _load_data(
+            os.path.join("x509", "ocsp", "ocsp-army.deps.mil-resp.der"),
+            ocsp.load_der_ocsp_response,
+        )
+
         with pytest.raises(ValueError):
-            _load_data(
-                os.path.join("x509", "ocsp", "ocsp-army.deps.mil-resp.der"),
-                ocsp.load_der_ocsp_response,
-            )
+            resp.serial_number
+
+        assert len(list(resp.responses)) == 20
+
+    def test_multi_valued_responses(self):
+        req_valid = _load_data(
+            os.path.join("x509", "ocsp", "ocsp-army.valid-req.der"),
+            ocsp.load_der_ocsp_request,
+        )
+
+        req_revoked = _load_data(
+            os.path.join("x509", "ocsp", "ocsp-army.revoked-req.der"),
+            ocsp.load_der_ocsp_request,
+        )
+
+        req_irrelevant = _load_data(
+            os.path.join("x509", "ocsp", "ocsp-army.inapplicable-req.der"),
+            ocsp.load_der_ocsp_request,
+        )
+
+        resp = _load_data(
+            os.path.join("x509", "ocsp", "ocsp-army.deps.mil-resp.der"),
+            ocsp.load_der_ocsp_response,
+        )
+
+        for elem in resp.responses:
+            serial = elem.serial_number
+
+            assert req_irrelevant.serial_number != serial
+            if req_valid.serial_number == serial:
+                assert elem.issuer_key_hash == req_valid.issuer_key_hash
+                assert elem.issuer_name_hash == req_valid.issuer_name_hash
+                assert (
+                    elem.hash_algorithm.name == req_valid.hash_algorithm.name
+                )
+
+                assert elem.certificate_status == ocsp.OCSPCertStatus.GOOD
+
+                assert elem.this_update == datetime.datetime(2020, 2, 22, 0, 0)
+                assert elem.next_update == datetime.datetime(2020, 2, 29, 1, 0)
+            elif req_revoked.serial_number == serial:
+                assert elem.certificate_status == ocsp.OCSPCertStatus.REVOKED
+
+                assert (
+                    elem.revocation_reason
+                    == x509.ReasonFlags.cessation_of_operation
+                )
+                assert elem.revocation_time == datetime.datetime(
+                    2018, 5, 30, 14, 1, 39
+                )
 
     def test_load_unauthorized(self):
         resp = _load_data(
@@ -952,9 +1235,17 @@ class TestOCSPResponse(object):
         assert resp.signature_algorithm_oid == x509.ObjectIdentifier(
             "1.2.840.113549.1.1.2"
         )
-        with pytest.raises(UnsupportedAlgorithm):
+        with raises_unsupported_algorithm(None):
             resp.signature_hash_algorithm
 
+    def test_unknown_hash_algorithm(self):
+        resp = _load_data(
+            os.path.join("x509", "ocsp", "resp-unknown-hash-alg.der"),
+            ocsp.load_der_ocsp_response,
+        )
+        with raises_unsupported_algorithm(None):
+            resp.hash_algorithm
+
     def test_load_responder_key_hash(self):
         resp = _load_data(
             os.path.join("x509", "ocsp", "resp-responder-key-hash.der"),
@@ -989,7 +1280,20 @@ class TestOCSPResponse(object):
         ext = resp.extensions[0]
         assert ext.critical is False
         assert ext.value == x509.OCSPNonce(
-            b'\x04\x105\x957\x9fa\x03\x83\x87\x89rW\x8f\xae\x99\xf7"'
+            b'5\x957\x9fa\x03\x83\x87\x89rW\x8f\xae\x99\xf7"'
+        )
+
+    def test_response_unknown_extension(self):
+        resp = _load_data(
+            os.path.join("x509", "ocsp", "resp-unknown-extension.der"),
+            ocsp.load_der_ocsp_response,
+        )
+        assert len(resp.extensions) == 1
+        ext = resp.extensions[0]
+        assert ext.critical is False
+        assert ext.value == x509.UnrecognizedExtension(
+            x509.ObjectIdentifier("1.3.6.1.5.5.7.48.1.2.200"),
+            b'\x04\x105\x957\x9fa\x03\x83\x87\x89rW\x8f\xae\x99\xf7"',
         )
 
     def test_serialize_reponse(self):
@@ -1011,10 +1315,6 @@ class TestOCSPResponse(object):
         with pytest.raises(ValueError):
             resp.public_bytes(serialization.Encoding.PEM)
 
-    @pytest.mark.supported(
-        only_if=lambda backend: (backend._lib.Cryptography_HAS_SCT),
-        skip_message="Requires CT support",
-    )
     def test_single_extensions_sct(self, backend):
         resp = _load_data(
             os.path.join("x509", "ocsp", "resp-sct-extension.der"),
@@ -1032,27 +1332,6 @@ class TestOCSPResponse(object):
             b"7ku9t3XOYLrhQmkfq+GeZqMPfl+wctiDAMR7iXqo/cs=",
         ]
 
-    @pytest.mark.supported(
-        only_if=lambda backend: (
-            not backend._lib.CRYPTOGRAPHY_OPENSSL_110F_OR_GREATER
-        ),
-        skip_message="Requires OpenSSL < 1.1.0f",
-    )
-    def test_skips_single_extensions_scts_if_unsupported(self, backend):
-        resp = _load_data(
-            os.path.join("x509", "ocsp", "resp-sct-extension.der"),
-            ocsp.load_der_ocsp_response,
-        )
-        with pytest.raises(x509.ExtensionNotFound):
-            resp.single_extensions.get_extension_for_class(
-                x509.SignedCertificateTimestamps
-            )
-
-        ext = resp.single_extensions.get_extension_for_oid(
-            x509.ExtensionOID.SIGNED_CERTIFICATE_TIMESTAMPS
-        )
-        assert isinstance(ext.value, x509.UnrecognizedExtension)
-
     def test_single_extensions(self, backend):
         resp = _load_data(
             os.path.join("x509", "ocsp", "resp-single-extension-reason.der"),
@@ -1063,8 +1342,35 @@ class TestOCSPResponse(object):
         assert ext.oid == x509.CRLReason.oid
         assert ext.value == x509.CRLReason(x509.ReasonFlags.unspecified)
 
+    def test_unknown_response_type(self):
+        with pytest.raises(ValueError):
+            _load_data(
+                os.path.join(
+                    "x509", "ocsp", "resp-response-type-unknown-oid.der"
+                ),
+                ocsp.load_der_ocsp_response,
+            )
 
-class TestOCSPEdDSA(object):
+    def test_response_bytes_absent(self):
+        with pytest.raises(ValueError):
+            _load_data(
+                os.path.join(
+                    "x509", "ocsp", "resp-successful-no-response-bytes.der"
+                ),
+                ocsp.load_der_ocsp_response,
+            )
+
+    def test_unknown_response_status(self):
+        with pytest.raises(ValueError):
+            _load_data(
+                os.path.join(
+                    "x509", "ocsp", "resp-unknown-response-status.der"
+                ),
+                ocsp.load_der_ocsp_response,
+            )
+
+
+class TestOCSPEdDSA:
     @pytest.mark.supported(
         only_if=lambda backend: backend.ed25519_supported(),
         skip_message="Requires OpenSSL with Ed25519 support / OCSP",
